-- treegrow (Minetest 0.4 mod)
-- Growing large trees

local TREE_GROW_DELAY = 7200 -- 2 hrs
local TREE_GROW_CHANCE = 3 -- 0.5%
local DENSITY = 3 -- allow 3 trunks in one 'cross'
local MINIMUM_LIGHT = 8 -- light needed for the trees to grow

local CACTUS_GROW_DELAY = 7200
local CACTUS_GROW_CHANCE = 3

local ABM_CHANCE = 50

-- Randomization
math.randomseed(os.time())

-- Tree growing
minetest.register_abm({
    nodenames = { "default:leaves" },
    interval = TREE_GROW_DELAY,
    chance = ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > TREE_GROW_CHANCE) then
            return
        end

        if(minetest.env:get_node_light(pos, nil) < MINIMUM_LIGHT) then
            return
        end
        
        local trunk_count = 0
        
        -- Check for trunks in area
        for i = -1, 1 do
        for j = -1, 0 do
        for k = -1, 1 do
            local current_node = {
                x = pos.x + i,
                y = pos.y + j,
                z = pos.z + k
            }
            if(minetest.env:get_node(current_node).name == "default:tree") and
            (math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                trunk_count = trunk_count + 1
            end
        end
        end
        end
        
        -- If there is at least 1 trunk and there are not many of them...
        if (trunk_count > 0) and (trunk_count < DENSITY) then
            minetest.env:remove_node(pos)
            minetest.env:add_node(pos, {name = "default:tree"})
            print ('[treegrow] A trunk has grown at (' .. pos.x .. ',' .. pos.y .. ',' .. pos.z .. ')')
            for i = -1, 1 do
            for j = -1, 1 do
            for k = -1, 1 do
                local current_node = {
                    x = pos.x + i,
                    y = pos.y + j,
                    z = pos.z + k
                }
                if(minetest.env:get_node(current_node).name == "air") then
                    minetest.env:add_node(current_node, {name = "default:leaves"})
                end
            end
            end
            end
        end
    end
})


-- Cactus growing
minetest.register_abm({
    nodenames = { "default:cactus" },
    interval = CACTUS_GROW_DELAY,
    chance = ABM_CHANCE,
    
    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > CACTUS_GROW_CHANCE) then
            return
        end
        -- Grow with 1/6 chance (choosing which side to grow)
        for i = -1, 1 do
        for j = 0, 1 do
        for k = -1, 1 do
            if(math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                if(math.random(1, 30) == 1) then
                    local new_pos = {
                        x = pos.x + i,
                        y = pos.y + j,
                        z = pos.z + k
                    }
                    if(minetest.env:get_node(new_pos).name == "air") then
                        minetest.env:add_node(new_pos, {name = "default:cactus"})
                        print ('[treegrow] A cactus has grown at (' .. pos.x .. ',' .. pos.y .. ',' .. pos.z .. ')')
                    end
                end
            end
        end
        end
        end
    end
})
